/*
 * Copyright (c) 2020 Olof Kindgren <olof.kindgren@gmail.com>
 * Copyright (c) 2024 Markus Koch <markus@notsyncing.net>
 *
 * SPDX-License-Identifier: Apache-2.0
 */

#include <zephyr/kernel.h>
#include <zephyr/arch/cpu.h>
#include <zephyr/device.h>
#include <zephyr/drivers/uart.h>
#include <inttypes.h>

#define UART0_BASE 0x81000000
#define UART0_SR (*((volatile uint32_t *) (UART0_BASE + 0x00)))
#define UART0_DR (*((volatile uint32_t *) (UART0_BASE + 0x04)))

#define UART0_SR_RX_DATA_EMPTY (1 << 0)
#define UART0_SR_TX_FULL (1 << 1)

#define DT_DRV_COMPAT trashernet_serial

static struct k_spinlock lock;

static void uart_trashernet_poll_out(const struct device *dev, unsigned char c)
{
	ARG_UNUSED(dev);

	k_spinlock_key_t key = k_spin_lock(&lock);

	while (UART0_SR & UART0_SR_TX_FULL);
	UART0_DR = c;

	k_spin_unlock(&lock, key);
}

static int uart_trashernet_poll_in(const struct device *dev, unsigned char *c)
{
	ARG_UNUSED(dev);

	*c = UART0_DR;

	return (UART0_SR & UART0_SR_RX_DATA_EMPTY) ? -1 : 0;
}

static int uart_trashernet_init(const struct device *dev)
{
  ARG_UNUSED(dev);
	return 0;
}

static const struct uart_driver_api uart_trashernet_driver_api = {
	.poll_in          = uart_trashernet_poll_in,
	.poll_out         = uart_trashernet_poll_out,
	.err_check        = NULL,
};

struct my_dev_data {

};

struct my_dev_cfg {

};

#define CREATE_MY_DEVICE(inst)                                       \
     static struct my_dev_data my_data_##inst = {                    \
     };                                                              \
     static const struct my_dev_cfg my_cfg_##inst = {                \
     };                                                              \
     DEVICE_DT_INST_DEFINE(inst,                                     \
                           uart_trashernet_init,                        \
                           NULL,                                     \
                           &my_data_##inst,                          \
                           &my_cfg_##inst,                           \
                           PRE_KERNEL_1, CONFIG_SERIAL_INIT_PRIORITY,  \
                           &uart_trashernet_driver_api);

DT_INST_FOREACH_STATUS_OKAY(CREATE_MY_DEVICE)
